<?php
/**
 * @copyright © 2008, 2009 Intrahealth International, Inc.
 * This File is part of I2CE
 *
 * I2CE is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/**
*  I2CE_SwissConfig_CustomReports_ReportView
* @package I2CE
* @subpackage Core
* @author Carl Leitner <litlfred@ibiblio.org>
* @version 2.1
* @access public
*/


class I2CE_Swiss_CustomReports_ReportView extends I2CE_Swiss_CustomReports_ReportView_Base{
    public function setReport($report) {
        $swissRep = $this->getSwissReport($report);
        if (!$swissRep instanceof I2CE_Swiss_CustomReports_Report){
            I2CE::raiseError("Invalid report $report");
            return false;
        }
        $this->setField('report',$report);
        return true;        
    }
    public function getReport() {
        if (!$this->hasField('report')) {
            return false;
        }
        return $this->getField('report');
    }

    public function hasLimitedView() {
        if (!$this->hasField('limit_view_to')) {
            return false;
        }
        return (strlen($this->getField('limit_view_to')) > 0);
    }

    public function setLimitedView($limited_view) {
        $this->setField('limit_view_to',$limited_view);
    }

    public function getLimitedView() {
        if (!$this->hasField('limit_view_to')) {
            return false;
        }
        return $this->getField('limit_view_to');    
    }

    public function setDisplayName($displayName) {
        $this->setTranslatableField('display_name',$displayName);
    }

    public function getDisplayName() {
        if ($this->hasField('display_name')) {
            return $this->getField('display_name');
        } else {
            return $this->humanText($this->name);
        }
    }

    public function hasDescription() {
        return $this->hasField('description');
    }

    public function getDescription() {
        if ($this->hasDescription()) {
            return $this->getField('description');
        } else {
            return false;
        }
    }

    public function setDescription($descr) {
        $this->setTranslatableField('description',$descr);
    }



    public function sortFields(&$fields) {
        $orders = $this->getOrder();
        $t_fields = array();
        foreach($orders as $order) {
            //first put all the fields that have been ordered
            if (!array_key_exists($order,$fields)) {
                continue;
            }
            $t_fields[$order]= $fields[$order];
            unset($fields[$order]);
        }
        while (count($fields) > 0) {
            //now put the remaining fields 
            reset($fields);
            $key = key($fields);
            $t_fields[$key] = array_shift($fields);
        }
        $fields = $t_fields;
    }

    public function setOrder($order) {
        if (is_string($order)) {
            $order = explode(',',$order);
        }
        if (!is_array($order)) {
            $order = array();
        }
        foreach ($order as &$ord) {
            $ord = trim($ord);
        }
        $this->setField('display_order',implode(',',$order));
        return true;
    }

    public function getOrder($as_array = true) {
        if ($this->hasField('display_order')) {
            if ($as_array) {
                return explode(",",$this->getField('display_order'));
            } else {
                return $this->getField('display_order');
            }
        } else {
            if ($as_array) {
                return array();
            } else {
                return false;
            }
        }
    }
    public function setRelatedViews($views) {
        if (is_string($views)) {
            $views = explode(',',$views);
        }
        if (!is_array($views)) {
            $views = array();
        }
        foreach ($views as &$view) {
            $view = trim($view);
        }
        $this->setField('related_views',implode(',',$views));
        return true;
    }


    public function getRelatedViews($as_array = true) {
        if ($this->hasField('related_views')) {
            if ($as_array) {
                return explode(",",$this->getField('related_views'));
            } else {
                $this->getField('related_views');
            }
        } else {
            if ($as_array) {
                return array();
            } else {
                return false;
            }
        }
    }


    public function hasTotal() {
        return ($this->hasField('total') && $this->getField('total') == 1);
    }

    public function setTotal($total) {
        if ($total) {
            $this->setField('total','1');
        } else {
            $this->setField('total','0');
        }
    }




    public function processValues($values) {
        if (!$this->page->hasPermission('task(custom_reports_can_edit_reportViews)')) {
            return false;
        }
        if (array_key_exists('total_hide',$values)) {
            if (array_key_exists('total',$values) && $values['total']) {
                $this->setTotal(true);
            } else {
                $this->setTotal(false);
            }
        }
        if (array_key_exists('display_order',$values)) {            
            $this->setOrder($values['display_order']);
        }
        if (array_key_exists('related_views',$values)) {
            $this->setRelatedViews($values['related_views']);
        }
        if (array_key_exists('description',$values)) {
            $this->setDescription($values['description']);
        }
        if (array_key_exists('displayName',$values)) {
            $this->setDisplayName($values['displayName']);
        }
        return true;
    }



    public function getChildType($child) {
        switch ($child) {
        case 'default_display_options':
            return 'CustomReports_ReportView_DisplayOptions';
        case 'fields':
            return 'CustomReports_ReportView_Fields';
        default:
            return parent::getChildType($child);
        }
    }

    public function displayValues($contentNode, $transient_options, $action) {
        $mainNode = $this->template->appendFileByNode('customReports_reportView_' . $action .'.html','div',$contentNode);                
        if (!$mainNode instanceof DOMNode) {
            I2CE::raiseError("Could not load report views template");
            return false;
        }        
        $this->template->setDisplayDataImmediate('displayName',$this->getDisplayName(),$mainNode);
        $this->template->setDisplayDataImmediate('description',$this->getDescription(),$mainNode);
        $this->template->setDisplayDataImmediate('show_link',$this->getURLRoot('show'). '/../' . $this->name,$mainNode);        
        $this->template->setDisplayDataImmediate('total',$this->hasTotal(),$mainNode);
        $all_views = array();
        $swissReportViews = $this->getAncestorByClass('I2CE_Swiss_CustomReports_ReportViews');
        if ($swissReportViews instanceof I2CE_Swiss_CustomReports_ReportViews) {
            foreach ($swissReportViews as $view=>$swissReportView) {
                if ($view == $this->name) {
                    continue;
                }
                $all_views[$view] = $swissReportView->getDisplayName();
            }
        }
        $this->template->setDisplayDataImmediate('related_views[]',$all_views,$mainNode);
        $this->template->selectOptionsImmediate('related_views[]',$this->getRelatedViews(),$mainNode);
        $this->renameInputs(array('displayName','description','total','total_hide','related_views[]'),$mainNode);
        $swissFields = $this->getChild('fields',true);
        if ($swissFields instanceof I2CE_Swiss_CustomReports_ReportView_Fields) {
            $fieldHeaders = array();
            foreach ($swissFields as $field=>$swissField) {
                $fieldHeaders[$field] = $swissField->getHeader();
            }
            $this->sortFields($fieldHeaders);                       
            $fieldsNode = $this->template->getElementById('fields_fill', $mainNode);
            if (!$fieldsNode instanceof DOMNode) {
                I2CE::raiseError("Dont know where to put fields");
                return false;
            }
            $transient_options['field_order'] = array_keys($fieldHeaders);
            $swissFields->addLink('fields_content','fields_fill',$mainNode,$action,$transient_options);
            $this->addFieldSorter($fieldHeaders,$fieldsNode);
        }
        return true;
    }



    protected function addFieldSorter($fieldHeaders,$mainNode) {
        $this->template->addHeaderLink('mootools-core.js');
        $this->template->addHeaderLink('mootools-more.js');
        $inputNode =  $this->template->createElement('input',array('name'=>'display_order','type'=>'hidden','value'=>implode(',',array_keys($fieldHeaders))));
        $this->template->appendNode($inputNode,        $mainNode);
        $ret = $this->renameInputs('display_order',$inputNode);
        if (!array_key_exists('display_order',$ret)) {
            I2CE::raiseError("Could not add sorter");
            return;
        }
        $inputNode->setAttribute('id',$ret['display_order']);
        $fields_id = 'fields_list:' . $this->path;
        $this->template->reIdNodes('fields_list', $fields_id, $mainNode);
        $display_order_name = $ret['display_order'];
        $js="
window.addEvent('domready',function() {
    var displayed_field_sorter = $('{$fields_id}');    
    var displayed_field_sort = $('{$display_order_name}');    
    if (displayed_field_sorter && displayed_field_sort ) { 
       var displayed_field_sortOptions = {
            handle:'span.sortablehandle',
            onComplete: function() {
                var order = new Array();;
                displayed_field_sorter.getElements('.sortme').each(function(e) {
                    order.push(e.get('text'));
                });
                displayed_field_sort.setProperty('value',order.join(','));
            },
            opacity: 0.5
       };
       new Sortables(displayed_field_sorter,displayed_field_sortOptions);
    }
});
";
        $this->template->addHeaderText($js,'script',true);

    }

}
# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
