<?php
/**
 * @copyright © 2007, 2009 Intrahealth International, Inc.
 * This File is part of I2CE
 *
 * I2CE is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/**
*  I2CE_Page_CachedForm
* @package I2CE
* @subpackage Core
* @author Carl Leitner <litlfred@ibiblio.org>
* @version 2.1
* @access public
*/


class I2CE_Page_CachedForm extends I2CE_Page{

    protected function addTextMessage($msg) {
        $this->template->getElementById('siteContent')->appendChild($this->template->createTextNode($msg));
    }


    /**
     * The business method if this page is called from the commmand line
     * @param array $request_remainder the remainder of the request after the page specfication.  
     * @param array $args the array of unix style command line arguments 
     * Arguements are link that in: http://us3.php.net/manual/en/features.commandline.php#78651
     * If we were called as: 
     *      index.php --page=/module/page/some/thing/else --long  -AB 2 -C -D 'ostrich' --eggs==good
     * Then $request_remainder = array('some','thing','else')
     * and $args = array('long'=>true, 'A'=>true, 'B'=>2, 'C'=>true, 'D'=>'ostrich', 'eggs'=>'good')
     */ 
    protected function actionCommandLine($args,$request_remainder)  {
        if (count($this->request_remainder) == 0) {
            if ( $this->page == 'cacheAll') {
                $this->cacheAll(true);
            } else if ($this->page == 'cacheAllForce') {
                $this->cacheAll(false);
            } else if ($this->page == 'dropAll') {
                $this->dropAll();
            } else {
                I2CE::raiseError("No action specfied");
            }
            return true;
        } else {
            $form = $this->request_remainder[0];
            try {
                $cachedForm = new I2CE_CachedForm($form);
            }
            catch(Exception $e) {
                $this->addTextMessage ( "Unable to setup cached form $form");
                return false;
            }
            $msg = '';
            switch ($this->page) {
            case 'createCache':            
                if ($cachedForm->generateCachedTable()) {
                    $msg =  "Cached Table for $form generated";
                } else {
                    $msg =  "Cached Table for $form not generated";
                }
                break;
            case 'forceCreateCache':            
                if ($cachedForm->generateCachedTable(false)) {
                    $msg =  "Cached Table for $form generated (forced)";
                } else {
                    $msg =  "Cached Table for $form not generated (forced)";
                }
                break;
            case 'dropCache':
                if ($cachedForm->dropTable()) {
                    $msg =  "Cached Table for $form dropped";
                } else {
                    $msg =  "Cached Table for $form not dropped";
                }
                break;
            default:
                $msg = "No action specifed for $form.  The valid actions are 'createCache', 'forceCreateCache', and 'dropCache'";
                break;
            }
            I2CE::raiseError($msg);
            return true;
        }
    }

    public function action() {
        parent::action();
        if (!$this->hasPermission('task(cached_forms_can_administer)')) {
            return false;
        }
        $this->template->setAttribute( "class", "active", "menuConfigure", "a[@href='configure']" );
        $this->template->appendFileById( "menu_configure.html", "ul", "menuConfigure" );
        $this->template->setAttribute( "class", "active", "menuCachedForms", "a[@href='CachedForms']" );
        if (count($this->request_remainder) == 0) {
            if ($this->page == 'cacheAll') {
                $this->cacheAll();
                $this->showMenu(null);
            } else if ($this->page == 'cacheAllForce') {
                $this->cacheAll(false);
                $this->showMenu(null);                  
            } else if ($this->page == 'dropAll') {              
                $this->dropAll();
                $this->showMenu(null);                                  
            } else {
                $this->showMenu($this->page);
            }
            return;
        }
        if (count($this->request_remainder) != 1) {
            $this->addTextMessage ( "No form specifed");
            return false;
        }
        $form = $this->request_remainder[0];
        try {
            $cachedForm = new I2CE_CachedForm($form);
        }
        catch(Exception $e) {
            $this->addTextMessage ( "Unable to setup cached form $form");
            return false;
        }
        $msg = '';
        switch ($this->page) {
        case 'createCache':            
            if ($cachedForm->generateCachedTable()) {
                $msg =  "Cached Table for $form generated";
            } else {
                $msg =  "Cached Table for $form not generated";
            }
            break;
        case 'forceCreateCache':            
            if ($cachedForm->generateCachedTable(false)) {
                $msg =  "Cached Table for $form generated (forced)";
            } else {
                $msg =  "Cached Table for $form not generated (forced)";
            }
            break;
        case 'dropCache':
            if ($cachedForm->dropTable()) {
                $msg =  "Cached Table for $form dropped";
            } else {
                $msg =  "Cached Table for $form not dropped";
            }
            break;
        default:
            $msg = "No action specifed for $form.  The valid actions are 'createCache', 'forceCreateCache', and 'dropCache'";
            break;
        }
        $this->addTextMessage($msg);
        return true;
    }


    protected function showMenu($action) {
        $factory = I2CE_FormFactory::instance();
        $forms = $factory->getForms();
        $contentNode = $this->template->getElementById('siteContent');
        if (!$contentNode instanceof DOMNode) {
            I2CE::raiseError("Dont know where to put cached form menu");
            return false;
        }
        $menuNode = $this->template->appendFileByNode("cachedforms_menu.html",'div',$contentNode);
        if (!$menuNode instanceof DOMNode) {
            I2CE::raiseError("Could not get main template for cached form menu");
            return false;
        }
        $ulNode = $this->template->getElementById('cached_forms_list',$menuNode);
        if (!$ulNode instanceof DOMNode ){
            I2CE::raiseError("Could not find where to add in cached forms");
            return false;
        }
        sort($forms);
        $timeConfig = I2CE::getConfig()->modules->CachedForms->times->generation;
        foreach ($forms as $form) {
            $formObj = $factory->createForm($form);
            if (!$formObj instanceof I2CE_Form) {
                I2CE::raiseError("Unable to instantiate form ($form)");
                continue;
            }
            $formNode = $this->template->appendFileByNode("cachedforms_menu_each.html",'li',$ulNode);
            if (!$formNode instanceof DOMNode) {
                I2CE::raiseError("Could not add in individual form");
                return false;
            }
            $time = 0;
            $timeConfig->setIfIsSet($time,$form);
            if ($time > 0) {
                $time = strftime('%c', $time)  ;
            } else {
                $time = '';
            }
            $this->template->setDisplayDataImmediate('form',$form,$formNode);
            $this->template->setDisplayDataImmediate('last_generated',$time,$formNode);
            $this->template->setDisplayDataImmediate('create_link',"CachedForms/createCache/$form", $formNode);
            $this->template->setDisplayDataImmediate('force_link',"CachedForms/forceCreateCache/$form", $formNode);
            $this->template->setDisplayDataImmediate('drop_link',"CachedForms/dropCache/$form", $formNode);
        }
        
    }


    /**
     * Attempts to drop caches of  all forms one by one. 
     */
    public function dropAll($forms = null) {
        I2CE::raiseError("Dropping all");
        $config = I2CE::getConfig()->modules->CachedForms;
        $factory = I2CE_FormFactory::instance();
        if ($forms === null) {
            $forms = $factory->getForms();
        }
        $failure = array();
        $config->cache_all->erase(); 
        foreach ($forms as $form) {
            try {
                $cachedForm = new I2CE_CachedForm($form);
            }
            catch(Exception $e) {
                if (array_key_exists('HTTP_HOST',$_SERVER)) { //we don't need to check here, b/c it won't error out.  we are doing it to keep the log file clean
                    $this->userMessage ( "Unable to setup cached form $form");
                }
                $sucess[] = $form;
                continue;
            }
            if (!$cachedForm->dropTable()) {
                $failure[] = $form;
            }
        }
        if (count($failure) > 0) {
            I2CE::raiseError("DropAll: Could not drop " . implode(',',$failure));
            if (array_key_exists('HTTP_HOST',$_SERVER)) {
                $this->userMessage( "Drop Failed:" . implode(',',$failure));
            }
        }  else {
            $this->userMessage( "Dropped all tables");
        }
        if (array_key_exists('HTTP_HOST',$_SERVER)) {
            $this->redirect("CachedForms");
        }
    }

    /**
     * Attempts to caches all forms one by one. 
     */
    public function cacheAll($not_force = true, $forms = null) {
        I2CE::raiseError("Caching all");
        $config = I2CE::getConfig()->modules->CachedForms;
        $factory = I2CE_FormFactory::instance();
        if ($forms === null) {
            $forms = $factory->getForms();
        }
        $sucess = array();
        $status = '';
        $config->cache_all->volatile(true);
        $config->setIfIsSet($status,'cache_all/status');
        if ( $status === 'in_progress' && $not_force) {
            I2CE::raiseError("In progress");
            return;
        }
        $config->cache_all->status == 'in_progress';
        foreach ($forms as $form) {
            try {
                $cachedForm = new I2CE_CachedForm($form);
            }
            catch(Exception $e) {
                if (array_key_exists('HTTP_HOST',$_SERVER)) { //we don't need to check here, b/c it won't error out.  we are doing it to keep the log file clean
                    $this->userMessage ( "Unable to setup cached form $form");
                }
                $sucess[] = $form;
                continue;
            }
            if ($cachedForm->generateCachedTable($not_force)) {
                $msg =  "Cached Table for $form generated";
            } else {
                $msg =  "Cached Table for $form not generated";
                $sucess[] = $form;
            }
            if (array_key_exists('HTTP_HOST',$_SERVER)) {
                $this->userMessage($msg,'notice');
            } else {
                I2CE::raiseError($msg);
            }
        }
        if (count($sucess) > 0) {
            I2CE::raiseError("CacheAll: Could not cache " . implode(',',$sucess));
            if (array_key_exists('HTTP_HOST',$_SERVER)) {
                $this->userMessage( "Could not cache " . implode(',',$sucess));
            }
        }
        $config->cache_all->status = 'done';
        $config->cache_all->time = time();
        if (array_key_exists('HTTP_HOST',$_SERVER)) {
            $this->redirect("CachedForms");
        }
        
    }

}
# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
