<?php
/**
* © Copyright 2009 IntraHealth International, Inc.
* 
* This File is part of I2CE 
* 
* I2CE is free software; you can redistribute it and/or modify 
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License 
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
* @package I2CE
* @subpackage core
* @author Carl Leitner <litlfred@ibiblio.org>
* @version v4.0.1
* @since v4.0.1
* @filesource 
*/ 
/** 
* Class I2CE_FormStorage_multi_flat
* 
* @access public
*/


class I2CE_FormStorage_multi_flat extends I2CE_FormStorage_DB {

    /**
     * Checks to see if this storage mechansim is writable
     * @returns boolean. false
     */
    public function isWritable() {
        return false;
    }




    protected function fieldIsColumn($data) {
        $col = '';
        $data->setIfIsSet($col,'col');
        $col = trim($col);
        return (strlen($col)>0);
    }



    protected function getFieldData($data) {
        $col = '';
        $data->setIfIsSet($col,'col');
        $col = trim($col);
        if (strlen($col)>0) {
            return $col;
        }
        $function = '';
        $data->setIfIsSet($function,'function');
        if (strlen($function)> 0) {
            return $function;
        }
        return false;
    }

    /**
     * @var protected array sting.  Keys are componentities and values are the assoicated database
     */
    protected $databases;

    /**
     * @var protected array componentized_forms. The array of componentized forms
     */
    protected $componentized_forms;



    /**
     * The constructor for the storage mechanism
     * @param string $name
     * @param array $global_options. Default to empty array. The array of options that are 
     * the same across all all forms which share a commone storage mechanisms
     */
    public function __construct($name, $global_options=array()) {
        parent::__construct($name,$global_options);
        $this->storage_options_cache[]  = array();
        $this->componentized_forms = I2CE_FormStorage::getComponentizedForms();
        if (!array_key_exists('components',$this->global_options) || !is_array($this->global_options['components'])) {
            I2CE::raiseError("No components specifed for multi-flat storage");
            return false;
        }
        $this->databases = array();
        foreach ($this->global_options['components'] as $component=>$data) {
            if (!is_array($data) || !array_key_exists('database',$data) || !is_string($data['database']) || strlen($data['database']) < 1) {
                continue;
            }
            $this->databases[$component] = $data['database'];
        }
    }

    /**
     * @var protected array $formObjs.  Keys are form names, values are instanceof I2CE_Form
     */ 
    protected $formObjs = array();

    /**
     * Get a (cached) form object for the given form
     * @param string $form
     * @returns I2CE_Form or false on failure.
     */ 
    protected function getFormObj($form) {
        if (!array_key_exists($form,$this->formObjs) ) {
            $formObj = I2CE_FormFactory::instance()->createForm($form);
            if (!$formObj instanceof I2CE_Form) {
                $formObj = false;
            }
            $this->formObjs[$form] = $formObj;
        }
        return $this->formObjs[$form];
    }


    /**
     *  Construct a query (to be used as a sub-select) to view the fields of the given form.  It always will return the id of the form as well
     *  @param string $form
     *  @param mixed $fields.  Either a string, the field, or an array of string, the fields.
     *  @param mixed $id.  Defaults to null.  If non-null it is the id that we wish to limit to.
     *  @param boolean $parent. Defaults to false.  If true, we include the parent id as a referenced field
     *  If it is scalar and non-boolean, it is consider to be the ID of the parent, and then we get all forms with parent the given id.
     *  @param callback $field_refernece_callback.  A callback function whose first arguement is the form, the second arguements
     *  is the field and which returns the way the field value should be references as a field.  If the callback is null (the default) then
     *  the reference used is "$form+$field"
     *  @param integer $mod_time. Defaults to -1.  If non-negative, we only list the requested fields for an id if at least one of them has a modification
     *    time greater than or equal to $mod_time.  If the form storage has no way of tracking modifucation time, all entries are listed.
     *  @returns string the query or false on failed.
     */
    public  function getRequiredFieldsQuery($form,$fields, $id=null, $parent = false, $field_reference_callback = null, $mod_time = -1)  {
        if ($fields === null) {
            $fields = array();
        } 
        if (is_string($fields)) {
            $fields = array($fields);
        }
        if (!is_array($fields)) {
            I2CE::raiseError("Invalid fields");
            return false;
        }
        if ( !($options = $this->getStorageOptions($form)) ) {            
            I2CE::raiseError("No valid multi_flat storage options for $form");
            return false;
        }
        if (!is_array($this->databases) || count($this->databases) == 0) {
            I2CE::raiseError("No databases specified");
            return false;
        }        
        $formObj = $this->getFormObj($form);
        if (!$formObj instanceof I2CE_Form) {
            I2CE::raiseError("Could not instantiate form $form");
            return false;
        }
        if (!in_array($form,$this->componentized_forms)) {
            I2CE::raiseError("Form $form is not specified as being componentized.  Valid componentized forms:" . print_r($this->componentized_forms,true));
            return false;
        }
        $table = '';
        $options->setIfIsSet($table,'table');
        $table = trim($table);
        if (strlen($table) == 0) {
            if (array_key_exists('table_prefix', $this->global_options) && is_string($this->global_options['table_prefix']) && strlen(trim($this->global_options['table_prefix'])) > 0) {
                
                $table_prefix = trim($this->global_options['table_prefix']);
            } else {
                $table_prefix  = 'hippo_';
            }
            $table = '`' . $table_prefix . $form . '`';
        }
        if (strlen($table) == 0) {
            I2CE::raiseError("No table specified for $form");
            return false;
        }
        $id_ref = false;
        if ($field_reference_callback != null) {
            if ( !is_string($id_ref = call_user_func($field_reference_callback, $form,'id')) ) {
                I2CE::raiseError("Invalid field reference callback function");
                return false;
            }                
        } else {
            $id_ref = "`$form+id`";
        }
        if (!$options->is_parent('id') || ($id_qry = $this->getFieldData($options->id)) === false) {
            $id_qry = 'id'; //default to column 'id'
        }
        $form_prepended = true;
        $options->setIfIsSet($form_prepended,"id/form_prepended");
        $unions = array();
        $p_componentized = false;
        if (is_scalar($id)) {
            if ( ($pos = strrpos($id,'@')) === false) { //get the last @ sign
                I2CE::raiseError("No component specified in id: $id ");
                return false;
            }
            $component = substr($id,$pos+1);
            $id_no_comp = substr($id,0,$pos);
            if ( strlen($component) == 0) {
                I2CE::raiseError("zero length component specified in id:$id");
                return false;
            }
            if (!array_key_exists($component,$this->databases)) {
                I2CE::raiseError("Component $component is not associated to a database in id $id");
                return false;
            }
            $databases = array($component => $this->databases[$component]);
            $componentParentForms = array_intersect(I2CE_Form::getAllowedParentForms($form), $this->componentized_forms);        
        } else if ( !is_bool($parent) && is_scalar($parent)) {
            list($p_form,$p_id) = explode('|',$parent,2);
            if (strlen($p_form) == 0) {
                I2CE::raiseError("No parent form given in $parent");
                return false;
            }
            if (strlen($p_id) == 0) {
                I2CE::raiseError("No parent id given in $parent");
                return false;
            }
            if (in_array($p_form,$this->componentized_forms)) {
                $p_componentized = true;
                if ( ($pos = strrpos($p_id,'@')) === false) { //get the last @ sign
                    I2CE::raiseError("No component specified in parent id: $parent");
                    return false;
                }
                $component = substr($p_id,$pos+1);
                $p_id_no_comp = substr($p_id,0,$pos);
                if ( strlen($component) == 0) {
                    I2CE::raiseError("zero length component specified in parent id:$parent");
                    return false;
                }
                if (!array_key_exists($component,$this->databases)) {
                    I2CE::raiseError("Component $component is not associated to a database in parent id: $parent");
                    return false;
                }
                $databases = array($component => $this->databases[$component]);
                $componentParentForms = array($p_form);        
            } else {
                $componentParentForms = array_intersect(I2CE_Form::getAllowedParentForms($form), $this->componentized_forms);                        
            }
            
        } else{ 
            $databases= $this->databases;
            $componentParentForms = array_intersect(I2CE_Form::getAllowedParentForms($form), $this->componentized_forms);        
        }

        //now verify that the databases and associated table are indeed present:
        $db = MDB2::singleton();
        
        foreach ($databases as $component=>$database) {
            $check_qry = "SELECT null FROM information_schema.table WHERE table_schema = '" . addslashes($database) . "'"
                . " AND table_name = '" . addslashes($table) . "'";
            $result = $db->query($check_qry);
            if (I2CE::pearError($result,"Cannot execute  query:\n$check_qry")) {
                return false;
            }
            if ($result->numRows() > 0) {
                //the table exists.
                continue;                
            }
            unset($databases[$component]);
        }
        if (count($databases) == 0) {
            I2CE::raiseError("No databases defined for multi-flat formstorage on table $table");
            return false;
        }

        foreach ($databases as $component=>$database) {
            $select_list = array();
            if ($form_prepended) {
                $select_list[] =   "CONCAT( SUBSTRING(" . $id_qry . "," .  (strlen($form) + 2) . "),'@','" . mysql_real_escape_string($component)  . "') AS $id_ref";
            } else {
                $select_list[] =   "CONCAT( " . $id_qry . ",'@','" . mysql_real_escape_string($component)  . "') AS $id_ref";
            }
            foreach ($formObj as $field=>$fieldObj) {
                if (!in_array($field,$fields)) {
                    continue;
                }
                if (!$fieldObj->isInDB()) {
                    continue;
                }
                $f_qry = false;
                if ($options->is_parent("fields/$field")) {
                    $data = $options->traverse("fields/$field");
                    if ($data->is_scalar('enabled') && !$data->enabled) {
                        continue; 
                    }
                    $f_qry = $this->getFieldData($data);
                }
                if ($f_qry === false) {
                    $f_qry = "`$field`"; //default to the field name
                }
                if ($fieldObj instanceof I2CE_FormField_MAPPED) {
                    $comp_map_forms = array_intersect($fieldObj->getSelectableForms(),$this->componentized_forms);
                    $f_qry = $fieldObj->getSQLComponentization($f_qry,$comp_map_forms,$component);
                }
                $f_ref = false;
                if ($field_reference_callback != null) {
                    if ( !is_string($f_ref = call_user_func($field_reference_callback, $form,$field)) ) {
                        I2CE::raiseError("Invalid field reference callback function");
                        return false;
                    }                
                } else {
                    $f_ref = "`$form+$field`";
                }
                $select_list[] = "$f_qry AS $f_ref";                
            } 
            $p_where='';
            if ($parent !== false) {
                $p_qry = false;
                $p_ref = false;
                if ($field_reference_callback != null) {
                    if ( !is_string($p_ref = call_user_func($field_reference_callback, $form,'parent')) ) {
                        I2CE::raiseError("Invalid field reference callback function");
                        return false;
                    }                
                } else {
                    $p_ref = "$form+parent";
                }
                if (!$options->is_scalar('parent/enabled') || $options->parent->enabled ) {
                    if ($options->is_parent('parent')) {
                        $p_qry = $this->getFieldData($options->parent);
                    }
                    if ($p_qry === false)  {
                        $p_qry = 'parent';
                    }
                } else {
                    $p_qry = '0';
                }                
                $select_list[] = I2CE_List::componentizeQuery($p_qry,$componentParentForms,$component). " AS `$p_ref`";
                if (!is_bool($parent) && is_scalar($parent)) {
                    if (!$p_componentized) {
                        $p_where = I2CE_List::componentizeQuery($p_qry,$componentParentForms,$component) . " = '" . mysql_real_escape_string($parent) . "'";
                    } else {
                        $p_where = "$p_qry = '" . mysql_real_escape_string($p_form . '|' . $p_id_no_comp) . "'";
                    }
                }            
            }
            
            //want either select id as `form+id`, name as `form+name` from table
            //or select id as `form+id`, name as `form+name` from (select * from table where id=5)
            //select  substr(my_id,1,4) as `form+id`, surname as `form+name` from (select * from table where id=5)
            //or with a function..
            //select id as `form+id`, substr(surname,1,3) as `form+name` from table
            //select  id as `form+id`, substr(surname,1,3) as `form+name` from (select * from table where id=5)
            //select  substr(my_id,1,4) as `form+id`, surname as `form+name` from table wher `form+id`
            
            if (is_scalar($id)) {                
//                 if (!$options->is_parent('id')) {
//                     $qry = '(SELECT ' . implode(',',$select_list) . "FROM (SELECT * FROM `$database`.$table WHERE id=$id LIMIT 1) AS `$form`)";
//                 } else { $this->fieldIsColumn($options->id)) {
//                     $qry = '(SELECT ' . implode(',',$select_list) . "FROM (SELECT * FROM `$database`.$table WHERE " . $options->id->col . "=$id LIMIT 1) AS `$form`)";
//                 } else {
                    if ($p_where) {
                        $p_where = " AND $p_where";
                    }
                    if ($form_prepended) {
                        $qry = 'SELECT ' . implode(',', $select_list) . " FROM `$database`.$table WHERE ( $id_qry = '" .  mysql_real_escape_string($form . '|' . $id_no_comp) . "' $p_where) ";
                    } else {
                        $qry = 'SELECT ' . implode(',', $select_list) . " FROM `$database`.$table WHERE ( $id_qry = '" .  mysql_real_escape_string($id_no_comp) . "' $p_where) ";
                    }
//                }
            } else {
                $qry = 'SELECT ' . implode(',', $select_list) . " FROM `$database`.$table ";
                if ($p_where) {
                    $qry .= 'WHERE ' . $p_where;
                }
            }
            $unions[] = $qry;
        }
        if (count($databases) > 1) {
            foreach ($unions as &$union) {
                $union = '(' . $union . ')';
            }
            //I2CE::raiseError("QRY:" . implode('UNION' ,$unions));
            return  implode('UNION', $unions);
        } else {
            reset($unions);
            //I2CE::raiseError("QRY:" . current($unions));
            return current($unions);
        }
        
    }
    

}
# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
